#!/usr/bin/python

################################################################
#                                                              #
# Pcapteller - A packet manipulation & replay tool             #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2015 Encripto AS. All rights reserved.             #
#                                                              #
# Pcapteller is licensed under the FreeBSD license.            #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################


import sys, getopt
import libs.tellerout, libs.tellercfg
from datetime import datetime

import logging
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import *


__version__ = "0.1"
__author__ = "Juan J. Guelfo, Encripto AS (post@encripto.no)"


if __name__ == '__main__':

    try:
        options, args = getopt.getopt(sys.argv[1:], "f:i:s:v:", ["pcap=", "interface=", "star=", "victim="])

    except getopt.GetoptError, err:
        libs.tellerout.print_header(__version__, __author__)
        libs.tellerout.print_error("%s.\n" % str(err))
        sys.exit(1)

    if not options:
        libs.tellerout.help(__version__, __author__)

    pcap_file = None
    net_interface = None
    victim_sim = None
    victim_real = None

    for opt, arg in options:
        if opt in ("-f"):
            pcap_file = arg

        if opt in ("-i"):
            net_interface = arg

        if opt in ("-s"):
            victim_sim = arg.replace("\n", "").replace(" ", "")

        if opt in ("-v"):
            victim_real = arg.replace("\n", "").replace(" ", "")

    if not pcap_file or not net_interface or not victim_sim or not victim_real:
        libs.tellerout.print_header(__version__, __author__)
        libs.tellerout.print_error("You must provide all mandatory arguments.\n")
        sys.exit(1)

    try:
        libs.tellerout.print_header(__version__, __author__)
        if not libs.tellercfg.check_ip_address(victim_sim) or not libs.tellercfg.check_ip_address(victim_real):
            libs.tellerout.print_error("Victim IP address(es) not valid.\n")
            sys.exit(1)

        libs.tellerout.print_info("Reading PCAP file...")
        libs.tellercfg.check_file_exists(pcap_file)
        pf = rdpcap(pcap_file)

        total_packets = len(pf)
        libs.tellerout.print_ok("%s packet(s) found.\n" % (total_packets))
        libs.tellerout.print_info("Starting replay at %s...\n" % (datetime.now().strftime("%H:%M:%S")))

        packet_counter = 0
        error_counter = 0
        packets = []
        for packet in pf:
            try:
                if packet.haslayer(Ether):
                    del(packet[Ether].chksum)

                if packet.haslayer(UDP):
                    del(packet[UDP].chksum)

                if packet.haslayer(ICMP):
                    del(packet[ICMP].chksum)

                if packet.haslayer(TCP):
                    del(packet[TCP].chksum)

                if packet.haslayer(IP):
                    del(packet[IP].chksum)
                    if packet[IP].src == victim_real:
                        packet[IP].src = victim_sim

                    elif packet[IP].dst == victim_real:
                        packet[IP].dst = victim_sim

                packets.append(packet)
                packet_counter += 1
                sys.stdout.write("\r\033[1;34m[*]\033[1;m Processing %s of %s packet(s) | Error: %s packet(s)." % (packet_counter, total_packets, error_counter))
                sys.stdout.flush()

            except Exception as e:
                error_counter += 1

        print ""
        error_counter = 0
        try:
            libs.tellerout.print_info("Replaying packet(s) via %s..." % (net_interface))
            sendp(packets, iface=net_interface, verbose=0)

        except:
            error_counter += 1

        libs.tellerout.print_info("Replay complete. %s packet(s) returned error during the replay.\n" % (error_counter))
        libs.tellerout.print_info("Pcapteller finished at %s.\n" % (datetime.now().strftime("%H:%M:%S")))
        sys.exit()

    except KeyboardInterrupt:
        print ""
        libs.tellerout.print_warning("CTRL+C was pressed. Shutting down...\n")
        sys.exit()

    except Exception as e:
        libs.tellerout.print_error("Exception: %s.\n" % (e))
        sys.exit()
